import java.lang.reflect.*;

public class Run {
  public static void main(String[ ] args) {
    // Verificamos de que se haya especificado el nombre de la clase
    if (args.length==0) {
      System.err.println("Ud. ha olvidado indicar el nombre de su programa.");
      System.err.println("Para ejecutar su programa, ejecute el siguiente comando:");
      System.err.println("java Run <nombre programa> <arg1> <arg2> ...");
      System.exit(1);
    }
      
    // Suprimos cualquier extension .java o .class que venga en el nombre de la clase
    String className= args[0];
    if ( testSuffix(className,".java") )
      className= className.substring(0, className.length()-".java".length());
    else if ( testSuffix(className,".class") )
      className= className.substring(0, className.length()-".class".length());

    // Intentamos cargar la clase
    Class classClass= null;
    try {
      classClass= Class.forName(className);
    }
    catch (ClassNotFoundException excp) {
      System.err.println("No se encontro la clase "+className);
      System.err.println("Se necesita que su programa se encuentre en alguno de los");
      System.err.println("directorios de la variable CLASSPATH.");
      showFormat(className);
      System.exit(2);
    }
    // Ok, la clase fue encontrada

    // Revisamos que la clase tenga un constructor que no reciba argumentos
    // para poder llamar a newInstance()
    try {
      classClass.getDeclaredConstructor(new Class[0]);
    }
    catch (NoSuchMethodException excp) {
      constructorError(className);
    }

    Application appl= null;
    try {
      appl= (Application)classClass.newInstance();
    }
    catch (InstantiationException excp) {
      System.err.println("No se pudo ejecutar la aplicacion porque la clase "+className);
      System.err.println("fue definida como abstract o interface.");
      showFormat(className);
      System.exit(3);
    }
    catch (IllegalAccessException excp) {
      constructorError(className);
    }
    catch (ClassCastException excp) {
      System.err.println("No se pudo ejecutar la aplicacion porque la clase "+className);
      System.err.println("no fue definida con extends Application");
      showFormat(className);
    }

    // Vemos si la clase tiene un main con argumentos
    Method method= null;
    Object[] paramArray= null;

    try {
     Class[] paramClassArray= { args.getClass() };
     method= classClass.getDeclaredMethod("main", paramClassArray);
      // Construimos el arreglo de argumentos para la aplicacion
      String[ ] trueArgs= new String[args.length-1];
      for (int i=1; i<args.length; i++) trueArgs[i-1]= args[i];
      Object[] paramArray2= { trueArgs };
      paramArray= paramArray2;
    }
    catch (NoSuchMethodException excp) {
      // Tratamos de ver si existe main()
      try {
        method= classClass.getDeclaredMethod("main", new Class[0]);
        paramArray= new Object[0];
      }
      catch (NoSuchMethodException excp2) {
        System.err.println("No se pudo ejecutar la aplicacion porque no se encontro");
        System.err.println("un metodo main() o un metodo main(String[] args).");
        System.err.println("");
        showFormat(className);
        System.exit(5);
      }
    }

    // Invocamos la aplicacion
    try {
      method.invoke(appl, paramArray);
      // Exito
    }
    catch (Exception excp) {
      System.err.println(excp.getMessage());
      excp.printStackTrace(System.err);
      System.exit(6);
    }
  }   

  static boolean testSuffix(String s, String suffix) {
    return suffix.length()<=s.length() &&
           suffix.equals(s.substring(s.length()-suffix.length()));
  }

  static void showFormat(String className) {
    System.err.println("");
    System.err.println("Su programa debe tener la siguiente forma:");
    System.err.println("");
    System.err.println("public "+className+" extends Application {");
    System.err.println("  void main() { // o tambien: void main(String[] args)");
    System.err.println("    ...");
    System.err.println("  }");
    System.err.println("  // Otros procedimientos");
    System.err.println("  ...");
    System.err.println("}");
    System.err.println("");
    System.err.println("No olvide que el archivo se debe llamar "+className+".java");
  }

  static void constructorError(String className) {
    System.err.println("No se pudo ejecutar la aplicacion porque la clase "+className);
    System.err.println("no posee un constructor sin argumentos.  Esto ha ocurrido porque");
    System.err.println("Ud. ha definido un constructor para la clase "+className+" como");
    System.err.println("public "+className+"( ... ) { ... }");
    System.err.println("Suprima todos los constructores o bien agregue un constructor que");
    System.err.println("no reciba argumentos.");
    System.exit(4);
  }
}
